"""
App init
"""

# Standard Library
from importlib import metadata

# Django
from django.utils.translation import gettext_lazy as _

__version__ = metadata.version(distribution_name="aa-srp")
__title__ = _("Ship Replacement")

del metadata



"""
Migrate srp data from the built-in SRP module
"""

# Django
from django.core.management.base import BaseCommand
from django.utils.crypto import get_random_string

# Alliance Auth
from allianceauth.srp.models import SrpFleetMain

# Alliance Auth (External Libs)
from eveuniverse.models import EveType

# AA SRP
from aasrp.helper.character import get_user_for_character
from aasrp.managers import SrpManager
from aasrp.models import RequestComment, SrpLink, SrpRequest


def get_input(text):
    """
    Wrapped input to migrate srp data
    """

    return input(text)


class Command(BaseCommand):
    help = "Migrate srp data from the built-in SRP module"

    def _migrate_srp_data(self) -> None:
        """
        Migrate srp data from the built-in SRP module

        :return:
        :rtype:
        """

        srp_links_migrated = 0
        srp_links_skipped = 0
        srp_requests_migrated = 0
        srp_requests_skipped = 0

        self.stdout.write("Migrating SRP fleets ...")
        srp_fleets = SrpFleetMain.objects.all()

        for srp_fleet in srp_fleets:
            # Let's see if the creator is still valid
            # Returns None when the creators account has been deleted
            # and no sentinel user can be created or obtained
            # in this case, we cannot create the fleet again
            srp_fleet_creator = get_user_for_character(
                character=srp_fleet.fleet_commander
            )

            if srp_fleet_creator is not None:
                srp_fleet_commander = srp_fleet.fleet_commander
                srp_fleet_name = srp_fleet.fleet_name
                srp_fleet_doctrine = srp_fleet.fleet_doctrine
                srp_fleet_time = srp_fleet.fleet_time
                srp_fleet_aar_link = srp_fleet.fleet_srp_aar_link

                # Fix srp status
                srp_fleet_status = SrpLink.Status.ACTIVE
                if srp_fleet.fleet_srp_status == "Completed":
                    srp_fleet_status = SrpLink.Status.COMPLETED

                if srp_fleet.fleet_srp_code == "":
                    srp_fleet_status = SrpLink.Status.CLOSED

                    # Also fix the missing SRP code, we need it!
                    srp_fleet.fleet_srp_code = get_random_string(
                        length=8  # 8 chars only because it's an old SRP link
                    )

                srp_fleet_srp_code = srp_fleet.fleet_srp_code

                self.stdout.write(f"Migrating SRP fleet {srp_fleet_srp_code} …")

                try:
                    srp_link = SrpLink.objects.get(srp_code=srp_fleet_srp_code)

                    srp_links_skipped += 1
                except SrpLink.DoesNotExist:
                    srp_link = SrpLink()

                    srp_link.srp_name = srp_fleet_name
                    srp_link.srp_status = srp_fleet_status
                    srp_link.srp_code = srp_fleet_srp_code
                    srp_link.fleet_doctrine = srp_fleet_doctrine
                    srp_link.aar_link = srp_fleet_aar_link
                    srp_link.fleet_time = srp_fleet_time
                    srp_link.fleet_commander = srp_fleet_commander
                    srp_link.creator = srp_fleet_creator
                    srp_link.save()

                    # Mark migrated the SRP link as COMPLETED and save the object
                    srp_fleet.fleet_srp_status = "Completed"
                    srp_fleet.save()

                    srp_links_migrated += 1

                    # get the new srp link object
                    srp_link = SrpLink.objects.get(srp_code=srp_fleet_srp_code)

                self.stdout.write(
                    f"Migrating SRP requests for SRP fleet {srp_fleet_srp_code} ..."
                )
                srp_userrequests = srp_fleet.srpuserrequest_set.all()

                for srp_userrequest in srp_userrequests:
                    # Let's see if the creator is still valid
                    # Returns None when the creators' account has been deleted
                    # and no sentinel user can be created or obtained
                    # in this case, we cannot create the request again
                    srp_userrequest_creator = get_user_for_character(
                        character=srp_userrequest.character
                    )

                    if srp_userrequest_creator is not None:
                        srp_userrequest_killboard_link = srp_userrequest.killboard_link

                        try:
                            SrpRequest.objects.get(
                                killboard_link=srp_userrequest_killboard_link
                            )

                            srp_requests_skipped += 1
                        except SrpRequest.DoesNotExist:
                            srp_userrequest_additional_info = (
                                srp_userrequest.additional_info
                            )
                            srp_userrequest_payout = srp_userrequest.srp_total_amount
                            srp_userrequest_loss_amount = srp_userrequest.kb_total_loss

                            try:
                                srp_userrequest_ship = EveType.objects.get(
                                    name=srp_userrequest.srp_ship_name
                                )
                            except EveType.DoesNotExist:
                                srp_kill_link = SrpManager.get_kill_id(
                                    srp_userrequest_killboard_link
                                )

                                (
                                    ship_type_id,
                                    ship_value,
                                    victim_id,
                                ) = SrpManager.get_kill_data(srp_kill_link)

                                (
                                    srp_userrequest_ship,
                                    created_from_esi,
                                ) = EveType.objects.get_or_create_esi(id=ship_type_id)

                            srp_userrequest_post_time = srp_userrequest.post_time
                            srp_userrequest_request_code = get_random_string(length=16)
                            srp_userrequest_character = srp_userrequest.character
                            srp_userrequest_srp_link = srp_link

                            srp_userrequest_status = SrpRequest.Status.PENDING
                            if srp_userrequest.srp_status == "Approved":
                                srp_userrequest_status = SrpRequest.Status.APPROVED

                            if srp_userrequest.srp_status == "Rejected":
                                srp_userrequest_status = SrpRequest.Status.REJECTED

                            srp_request = SrpRequest()
                            srp_request.killboard_link = srp_userrequest_killboard_link
                            srp_request.request_status = srp_userrequest_status
                            srp_request.payout_amount = srp_userrequest_payout
                            srp_request.loss_amount = srp_userrequest_loss_amount
                            srp_request.ship = srp_userrequest_ship
                            srp_request.post_time = srp_userrequest_post_time
                            srp_request.request_code = srp_userrequest_request_code
                            srp_request.character = srp_userrequest_character
                            srp_request.creator = srp_userrequest_creator
                            srp_request.srp_link = srp_userrequest_srp_link
                            srp_request.save()

                            # add request info to comments
                            srp_request_comment = RequestComment()
                            srp_request_comment.comment = (
                                srp_userrequest_additional_info
                            )
                            srp_request_comment.srp_request = srp_request
                            srp_request_comment.comment_type = (
                                RequestComment.Type.REQUEST_INFO
                            )
                            srp_request_comment.creator = srp_userrequest_creator
                            srp_request_comment.save()

                            srp_requests_migrated += 1

        self.stdout.write("Migration finished.")
        self.stdout.write(f"SRP links migrated: {srp_links_migrated}")
        self.stdout.write(f"SRP links skipped: {srp_links_skipped}")
        self.stdout.write(f"SRP requests migrated: {srp_requests_migrated}")
        self.stdout.write(f"SRP requests skipped: {srp_requests_skipped}")

    def handle(self, *args, **options):
        """
        Ask before running ...

        :param args:
        :type args:
        :param options:
        :type options:
        :return:
        :rtype:
        """

        self.stdout.write(
            "Migrating SRP links and requests from the built-in SRP module."
        )

        self.stdout.write(
            self.style.WARNING(
                "Due to the nature of the built-in modules design, "
                "ALL SRP links in the built-in module will be marked as COMPLETED "
                "after their migration so they cannot be migrated again and we "
                "don't end up with duplicates!"
            )
        )

        self.stdout.write(
            self.style.ERROR("!!! Make sure you have NO pending SRP requests !!!")
        )

        user_input = get_input("Are you sure you want to proceed? (yes/no)?")

        if user_input == "yes":
            self.stdout.write("Starting migration. Please stand by.")
            self._migrate_srp_data()
            self.stdout.write(self.style.SUCCESS("Migration complete!"))
        else:
            self.stdout.write(self.style.WARNING("Aborted."))



"""
Notifications helper
"""

# pylint: disable=import-outside-toplevel

# Standard Library
from datetime import datetime

# Django
from django.contrib.auth.models import User
from django.urls import reverse
from django.utils import timezone

# Alliance Auth
from allianceauth.notifications import notify
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag
from app_utils.urls import site_absolute_url

# AA SRP
from aasrp import __title__
from aasrp.app_settings import (
    aa_discordnotify_installed,
    allianceauth_discordbot_installed,
    discordproxy_installed,
)
from aasrp.constants import DISCORD_EMBED_COLOR_MAP
from aasrp.models import Setting, SrpRequest

logger = LoggerAddTag(get_extension_logger(__name__), __title__)


def _aadiscordbot_send_private_message(
    user_id: int,
    title: str,
    message: str,
    embed_message: bool = True,
    level: str = "info",
) -> None:
    """
    Try to send a PM to a user on Discord via allianceauth-discordbot

    :param user_id:
    :type user_id:
    :param title:
    :type title:
    :param message:
    :type message:
    :param embed_message:
    :type embed_message:
    :param level:
    :type level:
    :return:
    :rtype:
    """

    if allianceauth_discordbot_installed():
        logger.debug(
            msg="allianceauth-discordbot is active, trying to send private message"
        )

        # Third Party
        from aadiscordbot.tasks import send_message
        from discord import Embed

        embed = Embed(
            title=str(title),
            description=message,
            color=DISCORD_EMBED_COLOR_MAP.get(level),
            timestamp=datetime.now(),
        )

        if embed_message is True:
            send_message(user_id=user_id, embed=embed)
        else:
            send_message(user_id=user_id, message=f"**{title}**\n\n{message}")
    else:
        logger.debug(
            msg=(
                "allianceauth-discordbot is not available on this "
                "system to send the private message"
            )
        )


def _aadiscordbot_send_channel_message(
    channel_id: int,
    title: str,
    message: str,
    embed_message: bool = True,
    level: str = "info",
) -> None:
    """
    Try to send a message to a channel on Discord via allianceauth-discordbot

    :param channel_id:
    :type channel_id:
    :param title:
    :type title:
    :param message:
    :type message:
    :param embed_message:
    :type embed_message:
    :param level:
    :type level:
    :return:
    :rtype:
    """

    if allianceauth_discordbot_installed():
        logger.debug(
            msg="allianceauth-discordbot is active, trying to send channel message"
        )

        # Third Party
        from aadiscordbot.tasks import send_message
        from discord import Embed

        embed = Embed(
            title=str(title),
            description=message,
            color=DISCORD_EMBED_COLOR_MAP.get(level),
            timestamp=datetime.now(),
        )

        if embed_message is True:
            send_message(channel_id=channel_id, embed=embed)
        else:
            send_message(channel_id=channel_id, message=f"**{title}**\n\n{message}")
    else:
        logger.debug(
            msg=(
                "allianceauth-discordbot is not available on this "
                "system to send the channel message"
            )
        )


def _discordproxy_send_private_message(
    user_id: int,
    title: str,
    message: str,
    embed_message: bool = True,
    level: str = "info",
):
    """
    Try to send a PM to a user on Discord via discordproxy
    (fall back to allianceauth-discordbot if needed)

    :param user_id:
    :type user_id:
    :param title:
    :type title:
    :param message:
    :type message:
    :param embed_message:
    :type embed_message:
    :param level:
    :type level:
    :return:
    :rtype:
    """

    # Third Party
    from discordproxy.client import DiscordClient
    from discordproxy.exceptions import DiscordProxyException

    client = DiscordClient()

    try:
        logger.debug(msg="Trying to send a direct message via discordproxy")

        if embed_message is True:
            # Third Party
            from discordproxy.discord_api_pb2 import Embed

            footer = Embed.Footer(text=str(__title__))
            embed = Embed(
                title=str(title),
                description=message,
                color=DISCORD_EMBED_COLOR_MAP.get(level),
                timestamp=timezone.now().isoformat(),
                footer=footer,
            )

            client.create_direct_message(user_id=user_id, embed=embed)
        else:
            client.create_direct_message(
                user_id=user_id, content=f"**{title}**\n\n{message}"
            )
    except DiscordProxyException as ex:
        # Something went wrong with discordproxy
        # Fail silently and try if allianceauth-discordbot is available
        # as a last ditch effort to get the message out to Discord
        logger.debug(
            msg=(
                "Something went wrong with discordproxy, "
                "cannot send a direct message, trying allianceauth-discordbot "
                f"to send the message if available. Error: {ex}"
            )
        )

        _aadiscordbot_send_private_message(
            user_id=user_id,
            level=level,
            title=title,
            message=message,
            embed_message=embed_message,
        )


def _discordproxy_send_channel_message(
    channel_id: int,
    title: str,
    message: str,
    embed_message: bool = True,
    level: str = "info",
):
    """
    Try to send a message to a channel on Discord via discordproxy
    (fall back to allianceauth-discordbot if needed)

    :param channel_id:
    :type channel_id:
    :param title:
    :type title:
    :param message:
    :type message:
    :param embed_message:
    :type embed_message:
    :param level:
    :type level:
    :return:
    :rtype:
    """

    # Third Party
    from discordproxy.client import DiscordClient
    from discordproxy.exceptions import DiscordProxyException

    client = DiscordClient()

    try:
        logger.debug(msg="Trying to send a channel message via discordproxy")

        if embed_message:
            # Third Party
            from discordproxy.discord_api_pb2 import Embed

            footer = Embed.Footer(text=str(__title__))

            embed = Embed(
                title=title,
                description=message,
                color=DISCORD_EMBED_COLOR_MAP.get(level),
                timestamp=timezone.now().isoformat(),
                footer=footer,
            )
            client.create_channel_message(channel_id=channel_id, embed=embed)
        else:
            client.create_channel_message(
                channel_id=channel_id, content=f"**{title}**\n\n{message}"
            )
    except DiscordProxyException as ex:
        # Something went wrong with discordproxy
        # Fail silently and try if allianceauth-discordbot is available
        # as a last ditch effort to get the message out to Discord
        logger.debug(
            msg=(
                "Something went wrong with discordproxy, "
                "cannot send a channel message, "
                f"trying allianceauth-discordbot to send the message. Error: {ex}"
            )
        )

        _aadiscordbot_send_channel_message(
            channel_id=channel_id,
            level=level,
            title=title,
            message=message,
            embed_message=True,
        )


def send_user_notification(
    user: User,
    title: str,
    message: str,
    embed_message: bool = True,
    level: str = "info",
) -> None:
    """
    Send notification to user
    This creates a notification in Auth and a PM in Discord when either
    Discordproxy, AA-Discordbot or AA Discord Notifications is installed

    :param user:
    :type user:
    :param title:
    :type title:
    :param message:
    :type message:
    :param embed_message:
    :type embed_message:
    :param level:
    :type level:
    :return:
    :rtype:
    """

    getattr(notify, level)(user=user, title=title, message=message)

    # Handle Discord PMs when aa_discordnotify is not active
    # Check if either allianceauth_discordbot or discordproxy are available
    # to send the PM
    if hasattr(user, "discord"):  # Check if the user has a Discord account
        logger.debug(msg="User has a Discord account")

        # Check if discordnotify is active
        if not aa_discordnotify_installed():
            if discordproxy_installed():
                logger.debug(msg="discordproxy seems to be available ...")

                _discordproxy_send_private_message(
                    user_id=int(user.discord.uid),
                    level=level,
                    title=title,
                    message=message,
                    embed_message=embed_message,
                )
            else:
                # discordproxy not available, try if allianceauth-discordbot is
                # available
                logger.debug(
                    msg=(
                        "discordproxy not available to send a direct message, "
                        "let's see if we can use allianceauth-discordbot if available"
                    )
                )

                _aadiscordbot_send_private_message(
                    user_id=int(user.discord.uid),
                    level=level,
                    title=title,
                    message=message,
                    embed_message=embed_message,
                )
        else:
            logger.debug(
                msg="discordnotify is active, we don't have to send the PM ourself."
            )
    else:
        logger.debug(
            msg="User doesn't have a Discord account, can't send any messages ..."
        )


def send_message_to_discord_channel(
    channel_id: int, title: str, message: str, embed_message: bool = True
) -> None:
    """
    Sending a message to a discord channel.
    This creates a message to the SRP Team channel on Discord when either
    Discordproxy or AA-Discordbot is installed

    :param channel_id:
    :type channel_id:
    :param title:
    :type title:
    :param message:
    :type message:
    :param embed_message:
    :type embed_message:
    :return:
    :rtype:
    """

    # Check if either allianceauth_discordbot or discordproxy are available
    # to send the channel message
    if discordproxy_installed():
        logger.debug(msg="discordproxy seems to be available, check if we can use it")

        _discordproxy_send_channel_message(
            channel_id=channel_id,
            level="info",
            title=title,
            message=message,
            embed_message=embed_message,
        )
    else:
        # discordproxy not available, try if allianceauth-discordbot is available
        logger.debug(
            msg=(
                "discordproxy not available to send the channel message, "
                "let's see if we can use allianceauth-discordbot"
            )
        )

        _aadiscordbot_send_channel_message(
            channel_id=channel_id,
            level="info",
            title=title,
            message=message,
            embed_message=embed_message,
        )


def notify_srp_team(srp_request: SrpRequest, additional_info: str):
    """
    Send SRP request notification to the SRP teams Discord channel

    :param srp_request:
    :type srp_request:
    :param additional_info:
    :type additional_info:
    :return:
    :rtype:
    """

    srp_team_discord_channel = Setting.objects.get_setting(
        Setting.Field.SRP_TEAM_DISCORD_CHANNEL_ID
    )

    if srp_team_discord_channel is not None:
        site_base_url = site_absolute_url()
        request_code = srp_request.request_code
        character_name = srp_request.character.character_name
        ship_type = srp_request.ship.name
        zkillboard_link = srp_request.killboard_link
        additional_information = additional_info.replace("@", "{@}")
        srp_code = srp_request.srp_link.srp_code
        srp_link = site_base_url + reverse(
            viewname="aasrp:view_srp_requests", args=[srp_code]
        )

        title = "New SRP Request"
        message = f"**Request Code:** {request_code}\n"
        message += f"**Character:** {character_name}\n"
        message += f"**Ship:** {ship_type}\n"
        message += f"**zKillboard Link:** {zkillboard_link}\n"
        message += f"**Additional Information:**\n{additional_information}\n\n"
        message += f"**SRP Code:** {srp_code}\n"
        message += f"**SRP Link:** {srp_link}\n"

        logger.info(
            msg="Sending SRP request notification to the SRP team channel on Discord"
        )

        send_message_to_discord_channel(
            channel_id=srp_team_discord_channel,
            title=title,
            message=message,
        )



"""
Tests for the template tags
"""

# Django
from django.template import Context, Template
from django.test import TestCase

# Alliance Auth
from allianceauth.tests.auth_utils import AuthUtils

# AA SRP
from aasrp import __version__
from aasrp.models import get_sentinel_user
from aasrp.tests.utils import create_fake_user


class TestMainCharacterName(TestCase):
    """
    Tests for main_character_name template tag
    """

    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.template = Template(
            template_string="{% load aasrp_user %}{{ user|main_character_name }}"
        )

    def test_should_contain_character_name_for_users_with_main(self):
        """
        Should contain the character name for a user with a main set

        :return:
        :rtype:
        """

        # given
        user = create_fake_user(character_id=1001, character_name="Bruce Wayne")
        context = Context(dict_={"user": user})

        # when
        result = self.template.render(context=context)

        # then
        self.assertEqual(first=result, second="Bruce Wayne")

    def test_should_contain_user_character_name_for_users_without_main(self):
        """
        Should return the username for a users without a main character

        :return:
        :rtype:
        """

        # given
        user = AuthUtils.create_user(username="john")
        context = Context(dict_={"user": user})

        # when
        result = self.template.render(context=context)

        # then
        self.assertEqual(first=result, second="john")

    def test_should_return_deleted_user_for_sentinel_user(self):
        """
        Should return "deleted" for the sentinel user

        :return:
        :rtype:
        """

        # given
        user = get_sentinel_user()
        context = Context(dict_={"user": user})

        # when
        result = self.template.render(context=context)

        # then
        self.assertEqual(first=result, second="deleted")

    def test_should_be_empty_for_none(self):
        """
        Test should be empty

        :return:
        :rtype:
        """

        # given
        context = Context(dict_={"user": None})
        # when
        result = self.template.render(context=context)
        # then
        self.assertEqual(first=result, second="")


class TestMainCharacterId(TestCase):
    """
    Tests for main_character_id template tag
    """

    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.template = Template(
            template_string="{% load aasrp_user %}{{ user|main_character_id }}"
        )

    def test_should_contain_character_id_for_users_with_main(self):
        """
        Test should contain main character ID for users with main

        :return:
        :rtype:
        """

        # given
        user = create_fake_user(character_id=1001, character_name="Bruce Wayne")
        context = Context(dict_={"user": user})

        # when
        result = self.template.render(context=context)

        # then
        self.assertEqual(first=result, second="1001")

    def test_should_contain_dummy_id_for_users_without_main(self):
        """
        Test should contain a dummy ID (1) for users without a main character

        :return:
        :rtype:
        """

        # given
        user = AuthUtils.create_user(username="john")
        context = Context(dict_={"user": user})

        # when
        result = self.template.render(context=context)

        # then
        self.assertEqual(first=result, second="1")

    def test_should_be_dummy_id_for_sentinel_user(self):
        """
        Test should return a dummy ID (1) for the sentinel user

        :return:
        :rtype:
        """

        # given
        user = get_sentinel_user()
        context = Context(dict_={"user": user})

        # when
        result = self.template.render(context=context)

        # then
        self.assertEqual(first=result, second="1")

    def test_should_be_dummy_id_for_none(self):
        """
        Test should return a dummy ID (1) for None

        :return:
        :rtype:
        """

        # given
        context = Context(dict_={"user": None})

        # when
        result = self.template.render(context=context)

        # then
        self.assertEqual(first=result, second="1")


class TestMainCharacterCorporationName(TestCase):
    """
    Tests for main_character_corporation_name template tag
    """

    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.template = Template(
            template_string="{% load aasrp_user %}{{ user|main_character_corporation_name }}"
        )

    def test_should_contain_corp_name_for_users_with_main(self):
        """
        Test should return the corporation name for users with a main character

        :return:
        :rtype:
        """

        # given
        user = create_fake_user(
            character_id=1001,
            character_name="Bruce Wayne",
            corporation_id=2001,
            corporation_name="Wayne Tech Inc.",
            corporation_ticker="WYT",
        )
        context = Context(dict_={"user": user})

        # when
        result = self.template.render(context=context)

        # then
        self.assertEqual(first=result, second="Wayne Tech Inc.")

    def test_should_be_empty_for_users_without_main(self):
        """
        Test should be empty for users without a main character

        :return:
        :rtype:
        """

        # given
        user = AuthUtils.create_user(username="john")
        context = Context(dict_={"user": user})

        # when
        result = self.template.render(context=context)

        # then
        self.assertEqual(first=result, second="")

    def test_should_be_empty_for_sentinel_user(self):
        """
        Test should be empty for the sentinel user

        :return:
        :rtype:
        """

        # given
        user = get_sentinel_user()
        context = Context(dict_={"user": user})

        # when
        result = self.template.render(context=context)

        # then
        self.assertEqual(first=result, second="")

    def test_should_be_empty_for_none(self):
        """
        Test should be empty for None

        :return:
        :rtype:
        """

        # given
        context = Context(dict_={"user": None})

        # when
        result = self.template.render(context=context)

        # then
        self.assertEqual(first=result, second="")


class TestMainCorporationId(TestCase):
    """
    Tests for main_character_corporation_id template tag
    """

    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.template = Template(
            template_string="{% load aasrp_user %}{{ user|main_character_corporation_id }}"
        )

    def test_should_contain_corporation_id_for_users_with_main(self):
        """
        Test should return the main character's corp ID for users with a main character

        :return:
        :rtype:
        """

        # given
        user = create_fake_user(
            character_id=1001,
            character_name="Bruce Wayne",
            corporation_id=2001,
            corporation_name="Wayne Tech Inc.",
            corporation_ticker="WYT",
        )
        context = Context(dict_={"user": user})

        # when
        result = self.template.render(context=context)

        # then
        self.assertEqual(first=result, second="2001")

    def test_should_be_dummy_id_for_users_without_main(self):
        """
        Test should return a dummy ID (1) for users without a main character

        :return:
        :rtype:
        """

        # given
        user = AuthUtils.create_user(username="john")
        context = Context(dict_={"user": user})

        # when
        result = self.template.render(context=context)
        # then

        self.assertEqual(first=result, second="1")

    def test_should_be_dummy_id_for_sentinel_user(self):
        """
        Test should return a dummy ID (1) for the sentinel user

        :return:
        :rtype:
        """

        # given
        user = get_sentinel_user()
        context = Context(dict_={"user": user})

        # when
        result = self.template.render(context=context)

        # then
        self.assertEqual(first=result, second="1")

    def test_should_be_dummy_id_for_none(self):
        """
        Test should return a dummy ID (1) for None

        :return:
        :rtype:
        """

        # given
        context = Context(dict_={"user": None})

        # when
        result = self.template.render(context=context)

        # then
        self.assertEqual(first=result, second="1")


class TestMainCharacterAllianceName(TestCase):
    """
    Tests for main_character_alliance_name template tag
    """

    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.template = Template(
            template_string="{% load aasrp_user %}{{ user|main_character_alliance_name }}"
        )

    def test_should_contain_alliance_name_for_users_with_main(self):
        """
        Test should return the main character's alliance name for users with a main character

        :return:
        :rtype:
        """

        # given
        user = create_fake_user(
            character_id=1001,
            character_name="Bruce Wayne",
            corporation_id=2001,
            corporation_name="Wayne Tech Inc.",
            corporation_ticker="WYT",
            alliance_id=3001,
            alliance_name="Wayne Enterprices",
        )

        context = Context(dict_={"user": user})

        # when
        result = self.template.render(context=context)

        # then
        self.assertEqual(first=result, second="Wayne Enterprices")

    def test_should_be_empty_for_users_without_main(self):
        """
        Test should be empty for users without a main character

        :return:
        :rtype:
        """

        # given
        user = AuthUtils.create_user(username="john")
        context = Context(dict_={"user": user})

        # when
        result = self.template.render(context=context)

        # then
        self.assertEqual(first=result, second="")

    def test_should_be_empty_when_main_is_not_in_an_alliance(self):
        """
        Test should be empty when a main character is not in an alliance

        :return:
        :rtype:
        """

        # given
        user = create_fake_user(
            character_id=2012,
            character_name="William Riker",
            corporation_id=2012,
            corporation_name="Starfleet",
            corporation_ticker="SF",
            alliance_id=None,
        )

        context = Context(dict_={"user": user})

        # when
        result = self.template.render(context=context)

        # then
        self.assertEqual(first=result, second="")

    def test_should_be_empty_for_sentinel_user(self):
        """
        Test should be empty for the sentinel user

        :return:
        :rtype:
        """

        # given
        user = get_sentinel_user()
        context = Context(dict_={"user": user})

        # when
        result = self.template.render(context=context)

        # then
        self.assertEqual(first=result, second="")

    def test_should_be_empty_for_none(self):
        """
        Test should be empty for None

        :return:
        :rtype:
        """

        # given
        context = Context(dict_={"user": None})

        # when
        result = self.template.render(context=context)

        # then
        self.assertEqual(first=result, second="")


class TestMainAllianceId(TestCase):
    """
    Tests for main_character_alliance_id template tag
    """

    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.template = Template(
            template_string="{% load aasrp_user %}{{ user|main_character_alliance_id }}"
        )

    def test_should_contain_alliance_id_for_users_with_main(self):
        """
        Test should return the main character's alliance ID for user with a main character

        :return:
        :rtype:
        """

        # given
        user = create_fake_user(
            character_id=1001,
            character_name="Bruce Wayne",
            corporation_id=2001,
            corporation_name="Wayne Tech Inc.",
            corporation_ticker="WYT",
            alliance_id=3001,
            alliance_name="Wayne Enterprises",
        )
        context = Context(dict_={"user": user})

        # when
        result = self.template.render(context=context)

        # then
        self.assertEqual(first=result, second="3001")

    def test_should_be_dummy_id_for_users_without_main(self):
        """
        Test should return a dummy ID (1) for user without a main character

        :return:
        :rtype:
        """

        # given
        user = AuthUtils.create_user(username="john")
        context = Context(dict_={"user": user})

        # when
        result = self.template.render(context=context)

        # then
        self.assertEqual(first=result, second="1")

    def test_should_dummy_id_when_main_is_not_in_an_alliance(self):
        """
        Test should return a dummy ID (1) when the main character is not in an alliance

        :return:
        :rtype:
        """

        # given
        user = create_fake_user(
            character_id=2012,
            character_name="William Riker",
            corporation_id=2012,
            corporation_name="Starfleet",
            corporation_ticker="SF",
            alliance_id=None,
        )

        context = Context(dict_={"user": user})

        # when
        result = self.template.render(context=context)

        # then
        self.assertEqual(first=result, second="1")

    def test_should_be_dummy_id_for_sentinel_user(self):
        """
        Test should return a dummy ID (1) for the sentinel user

        :return:
        :rtype:
        """

        # given
        user = get_sentinel_user()
        context = Context(dict_={"user": user})

        # when
        result = self.template.render(context=context)

        # then
        self.assertEqual(first=result, second="1")

    def test_should_be_dummy_id_for_none(self):
        """
        Test should return a dummy ID (1) for None

        :return:
        :rtype:
        """

        # given
        context = Context(dict_={"user": None})

        # when
        result = self.template.render(context=context)

        # then
        self.assertEqual(first=result, second="1")


class TestForumVersionedStatic(TestCase):
    """
    Tests for aasrp_static template tag
    """

    def test_versioned_static(self):
        """
        Test should return the static URL string with the app version

        :return:
        :rtype:
        """

        context = Context(dict_={"version": __version__})
        template_to_render = Template(
            template_string=(
                "{% load aasrp_versioned_static %}"
                "{% aasrp_static 'aasrp/css/aa-srp.min.css' %}"
            )
        )

        rendered_template = template_to_render.render(context=context)

        self.assertInHTML(
            needle=f'/static/aasrp/css/aa-srp.min.css?v={context["version"]}',
            haystack=rendered_template,
        )



"""
Tests for model Setting
"""

# Django
from django.db.utils import IntegrityError
from django.test import TestCase

# AA SRP
from aasrp.models import Setting
from aasrp.tests.utils import create_setting


class TestSetting(TestCase):
    """
    Testing the Setting model
    """

    def test_model_string_name(self):
        """
        Test model string name

        :return:
        :rtype:
        """

        # given
        setting = Setting.objects.get(pk=1)

        # when/then
        self.assertEqual(first=str(setting), second="AA-SRP Settings")

    def test_setting_save(self):
        """
        Test if there can't be another setting created
        and the existing setting is changed instead

        :return:
        :rtype:
        """

        # given
        srp_team_discord_channel_id = 45698741256
        setting = Setting(pk=2, srp_team_discord_channel_id=srp_team_discord_channel_id)
        setting.save()

        # then
        self.assertEqual(first=setting.pk, second=1)
        self.assertEqual(
            first=setting.srp_team_discord_channel_id,
            second=srp_team_discord_channel_id,
        )

    def test_setting_create(self):
        """
        Test that create method throwing the following exception
        django.db.utils.IntegrityError: (1062, "Duplicate entry '1' for key 'PRIMARY'")

        :return:
        :rtype:
        """

        # No pk given
        with self.assertRaises(expected_exception=IntegrityError):
            create_setting()

    def test_setting_create_with_pk(self):
        """
        Test that create method throwing the following exception no matter the given pk
        django.db.utils.IntegrityError: (1062, "Duplicate entry '1' for key 'PRIMARY'")

        :return:
        :rtype:
        """

        # Set pk=2
        with self.assertRaises(expected_exception=IntegrityError):
            create_setting(pk=2)

    def test_cannot_be_deleted(self):
        """
        Test that the settings object cannot be deleted

        :return:
        :rtype:
        """

        # given
        settings_old = Setting.objects.get(pk=1)

        # when
        Setting.objects.all().delete()

        # then
        settings = Setting.objects.all()
        settings_first = settings.first()

        # See if there is still only ONE Setting object
        self.assertEqual(first=settings.count(), second=1)

        # Check if both of our objects are identical
        self.assertEqual(first=settings_old, second=settings_first)

    def test_srp_team_discord_channel_id_is_not_mandatory(self):
        """
        Test that we get None when the Discord Channel ID field is empty

        :return:
        :rtype:
        """

        # given
        setting = Setting()
        setting.save()

        # then
        self.assertEqual(first=setting.pk, second=1)
        self.assertEqual(first=setting.srp_team_discord_channel_id, second=None)



"""
Some helper functions, so we don't mess up other files too much
"""

# Django
from django.contrib.auth.decorators import login_required, permission_required
from django.core.handlers.wsgi import WSGIRequest
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

# Alliance Auth
from allianceauth.authentication.decorators import permissions_required

# AA SRP
from aasrp.models import SrpLink, SrpRequest


@login_required
@permission_required("aasrp.basic_access")
def get_dashboard_action_icons(request: WSGIRequest, srp_link: SrpLink) -> str:
    """
    Getting the action buttons for the dashboard view

    :param request:
    :type request:
    :param srp_link:
    :type srp_link:
    :return:
    :rtype:
    """

    actions = ""

    if srp_link.srp_status == SrpLink.Status.ACTIVE:
        button_request_url = reverse(
            viewname="aasrp:request_srp", args=[srp_link.srp_code]
        )
        btn_icon = '<i class="fas fa-hand-holding-usd"></i>'
        btn_title = _("Request SRP")
        actions += (
            f'<a href="{button_request_url}" '
            'class="btn btn-success btn-sm btn-icon-aasrp" '
            f'title="{btn_title}">{btn_icon}</a>'
        )

    if request.user.has_perm("aasrp.manage_srp") or request.user.has_perm(
        "aasrp.manage_srp_requests"
    ):
        button_view_url = reverse(
            viewname="aasrp:view_srp_requests", args=[srp_link.srp_code]
        )
        btn_icon = '<i class="fas fa-eye"></i>'
        btn_title = _("View SRP Requests")
        actions += (
            f'<a href="{button_view_url}" '
            'class="btn btn-primary btn-sm btn-icon-aasrp" '
            f'title="{btn_title}">{btn_icon}</a><br>'
        )

        if srp_link.srp_status != SrpLink.Status.COMPLETED:
            if request.user.has_perm("aasrp.manage_srp"):
                if srp_link.srp_status == SrpLink.Status.ACTIVE:
                    button_edit_url = reverse(
                        viewname="aasrp:edit_srp_link", args=[srp_link.srp_code]
                    )
                    btn_icon = '<i class="far fa-newspaper"></i>'
                    btn_title = _("Add/Change AAR Link")
                    actions += (
                        f'<a href="{button_edit_url}" '
                        'class="btn btn-info btn-sm btn-icon-aasrp" '
                        f'title="{btn_title}">{btn_icon}</a>'
                    )

                    button_disable_url = reverse(
                        viewname="aasrp:disable_srp_link", args=[srp_link.srp_code]
                    )
                    btn_icon = '<i class="fas fa-ban"></i>'
                    btn_title = _("Disable SRP Link")
                    modal_id = "disable-srp-link"
                    data_name = srp_link.srp_name + " (" + srp_link.srp_code + ")"

                    actions += (
                        '<a class="btn btn-warning btn-sm btn-icon-aasrp" '
                        f'title="{btn_title}" '
                        'data-toggle="modal" '
                        f'data-target="#{modal_id}" '
                        f'data-url="{button_disable_url}" '
                        f'data-name="{data_name}">{btn_icon}</a>'
                    )

                if srp_link.srp_status == SrpLink.Status.CLOSED:
                    button_enable_url = reverse(
                        viewname="aasrp:enable_srp_link", args=[srp_link.srp_code]
                    )
                    btn_icon = '<i class="fas fa-check"></i>'
                    btn_title = _("Enable SRP Link")
                    modal_id = "enable-srp-link"
                    data_name = srp_link.srp_name + " (" + srp_link.srp_code + ")"

                    actions += (
                        '<a class="btn btn-success btn-sm btn-icon-aasrp" '
                        f'title="{btn_title}" '
                        'data-toggle="modal" '
                        f'data-target="#{modal_id}" '
                        f'data-url="{button_enable_url}" '
                        f'data-name="{data_name}">{btn_icon}</a>'
                    )

                button_remove_url = reverse(
                    viewname="aasrp:delete_srp_link", args=[srp_link.srp_code]
                )
                btn_icon = '<i class="far fa-trash-alt"></i>'
                btn_title = _("Remove SRP Link")
                modal_id = "delete-srp-link"
                data_name = srp_link.srp_name + " (" + srp_link.srp_code + ")"

                actions += (
                    '<a class="btn btn-danger btn-sm btn-icon-aasrp" '
                    f'title="{btn_title}" '
                    'data-toggle="modal" '
                    f'data-target="#{modal_id}" '
                    f'data-url="{button_remove_url}" '
                    f'data-name="{data_name}">{btn_icon}</a>'
                )

    return actions


@login_required
@permission_required("aasrp.basic_access")
def get_srp_request_status_icon(request: WSGIRequest, srp_request: SrpRequest) -> str:
    """
    Get status icon for srp request

    :param request:
    :type request:
    :param srp_request:
    :type srp_request:
    :return:
    :rtype:
    """

    request_status_icon_title = _("SRP Request Pending")
    srp_request_status_icon = (
        '<button class="btn btn-info btn-sm btn-icon-aasrp btn-icon-aasrp-status" '
        f'title="{request_status_icon_title}">'
        '<i class="fas fa-clock"></i>'
        "</button>"
    )

    if srp_request.request_status == SrpRequest.Status.APPROVED:
        btn_classes = "btn btn-success btn-sm btn-icon-aasrp btn-icon-aasrp-status"
        request_status_icon_title = _("SRP Request Approved")
        srp_request_status_icon = (
            f'<button class="{btn_classes}" '
            f'title="{request_status_icon_title}">'
            '<i class="fas fa-thumbs-up"></i>'
            "</button>"
        )

    if srp_request.request_status == SrpRequest.Status.REJECTED:
        btn_classes = "btn btn-danger btn-sm btn-icon-aasrp btn-icon-aasrp-status"
        request_status_icon_title = _("SRP Request Rejected")
        srp_request_status_icon = (
            f'<button class="{btn_classes}" '
            f'title="{request_status_icon_title}">'
            '<i class="fas fa-thumbs-down"></i>'
            "</button>"
        )

    return srp_request_status_icon


@login_required
@permission_required("aasrp.basic_access")
def get_srp_request_details_icon(
    request: WSGIRequest, srp_link: SrpLink, srp_request: SrpRequest
) -> str:
    """
    Get details icon for an SRP request

    :param request:
    :type request:
    :param srp_link:
    :type srp_link:
    :param srp_request:
    :type srp_request:
    :return:
    :rtype:
    """

    button_request_details_url = reverse(
        viewname="aasrp:ajax_srp_request_additional_information",
        args=[srp_link.srp_code, srp_request.request_code],
    )

    title = _("SRP Request Details")

    srp_request_details_icon = (
        f'<button data-link="{button_request_details_url}" '
        'data-toggle="modal" '
        'data-target="#srp-request-details" '
        'class="btn btn-primary btn-sm btn-icon-aasrp" '
        f'title="{title}"><i class="fas fa-info-circle"></i></button>'
    )

    return srp_request_details_icon


@login_required
@permission_required("aasrp.basic_access")
def get_srp_request_accept_icon(
    request: WSGIRequest, srp_link: SrpLink, srp_request: SrpRequest
) -> str:
    """
    Get accept icon for an SRP request

    :param request:
    :type request:
    :param srp_link:
    :type srp_link:
    :param srp_request:
    :type srp_request:
    :return:
    :rtype:
    """

    button_request_accept_url = reverse(
        viewname="aasrp:ajax_srp_request_approve",
        args=[srp_link.srp_code, srp_request.request_code],
    )

    button_request_accept_state = ""
    if srp_request.request_status == SrpRequest.Status.APPROVED:
        button_request_accept_state = ' disabled="disabled"'

    modal_target = (
        "#srp-request-accept-rejected"
        if srp_request.request_status == SrpRequest.Status.REJECTED
        else "#srp-request-accept"
    )

    icon = '<i class="fas fa-check"></i>'
    title = _("Accept SRP Request")

    srp_request_accept_icon = (
        f'<button data-link="{button_request_accept_url}" '
        'data-toggle="modal" '
        f'data-target="{modal_target}" '
        'class="btn btn-success btn-sm btn-icon-aasrp" '
        f'title="{title}"{button_request_accept_state}>{icon}</button>'
    )

    return srp_request_accept_icon


@login_required
@permission_required("aasrp.basic_access")
def get_srp_request_reject_icon(
    request: WSGIRequest, srp_link: SrpLink, srp_request: SrpRequest
) -> str:
    """
    Get reject icon for an SRP request

    :param request:
    :type request:
    :param srp_link:
    :type srp_link:
    :param srp_request:
    :type srp_request:
    :return:
    :rtype:
    """

    button_request_reject_url = reverse(
        viewname="aasrp:ajax_srp_request_deny",
        args=[srp_link.srp_code, srp_request.request_code],
    )

    button_request_reject_state = ""
    if srp_request.request_status == SrpRequest.Status.REJECTED:
        button_request_reject_state = ' disabled="disabled"'

    icon = '<i class="fas fa-ban"></i>'
    title = _("Reject SRP Request")

    srp_request_reject_icon = (
        f'<button data-link="{button_request_reject_url}" '
        'data-toggle="modal" '
        'data-target="#srp-request-reject" '
        'class="btn btn-warning btn-sm btn-icon-aasrp" '
        f'title="{title}"{button_request_reject_state}>{icon}</button>'
    )

    return srp_request_reject_icon


@login_required
@permission_required("aasrp.basic_access")
def get_srp_request_delete_icon(
    request: WSGIRequest, srp_link: SrpLink, srp_request: SrpRequest
) -> str:
    """
    Get delete icon for an SRP request

    :param request:
    :type request:
    :param srp_link:
    :type srp_link:
    :param srp_request:
    :type srp_request:
    :return:
    :rtype:
    """

    button_request_delete_url = reverse(
        viewname="aasrp:ajax_srp_request_remove",
        args=[srp_link.srp_code, srp_request.request_code],
    )

    icon = '<i class="fas fa-trash-alt"></i>'
    title = _("Delete SRP Request")

    srp_request_delete_icon = (
        f'<button data-link="{button_request_delete_url}" '
        'data-toggle="modal" '
        'data-target="#srp-request-remove" '
        'class="btn btn-danger btn-sm btn-icon-aasrp" '
        f'title="{title}">{icon}</button>'
    )

    return srp_request_delete_icon


@login_required
@permissions_required(("aasrp.manage_srp_requests", "aasrp.manage_srp"))
def get_srp_request_action_icons(
    request: WSGIRequest, srp_link: SrpLink, srp_request: SrpRequest
) -> str:
    """
    Get action icons for srp requests

    :param request:
    :type request:
    :param srp_link:
    :type srp_link:
    :param srp_request:
    :type srp_request:
    :return:
    :rtype:
    """

    srp_request_action_icons = get_srp_request_details_icon(
        request=request, srp_link=srp_link, srp_request=srp_request
    )

    if srp_link.srp_status in (SrpLink.Status.ACTIVE, SrpLink.Status.CLOSED):
        # Accept
        srp_request_action_icons += get_srp_request_accept_icon(
            request=request, srp_link=srp_link, srp_request=srp_request
        )

        # Reject
        srp_request_action_icons += get_srp_request_reject_icon(
            request=request, srp_link=srp_link, srp_request=srp_request
        )

        # Delete
        if request.user.has_perm("aasrp.manage_srp"):
            srp_request_action_icons += get_srp_request_delete_icon(
                request=request, srp_link=srp_link, srp_request=srp_request
            )

    return srp_request_action_icons



"""
Form definitions
"""

# Standard Library
import re

# Django
from django import forms
from django.forms import ModelForm
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _

# AA SRP
from aasrp.constants import (
    EVETOOLS_KILLBOARD_BASE_URL,
    EVETOOLS_KILLBOARD_BASE_URL_REGEX,
    EVETOOLS_KILLBOARD_KILLMAIL_URL_REGEX,
    ZKILLBOARD_BASE_URL,
    ZKILLBOARD_BASE_URL_REGEX,
    ZKILLBOARD_KILLMAIL_URL_REGEX,
)
from aasrp.managers import SrpManager
from aasrp.models import FleetType, Setting, SrpLink, SrpRequest, UserSetting


def get_mandatory_form_label_text(text: str) -> str:
    """
    Label text for mandatory form fields

    :param text:
    :type text:
    :return:
    :rtype:
    """

    required_text = _("This field is mandatory")
    required_marker = (
        f'<span aria-label="{required_text}" class="form-required-marker">*</span>'
    )

    return mark_safe(
        f'<span class="form-field-required">{text} {required_marker}</span>'
    )


class SrpLinkForm(ModelForm):
    """
    New SRP lnk form
    """

    srp_name = forms.CharField(
        required=True, label=get_mandatory_form_label_text(text=_("Fleet Name"))
    )
    fleet_time = forms.DateTimeField(
        required=True,
        label=get_mandatory_form_label_text(text=_("Fleet Time")),
        widget=forms.DateTimeInput(attrs={"autocomplete": "off"}),
    )
    fleet_type = forms.ModelChoiceField(
        required=False,
        label=_("Fleet Type (optional)"),
        queryset=FleetType.objects.filter(is_enabled=True),
        # empty_label=_("Please select a fleet type"),
    )
    fleet_doctrine = forms.CharField(
        required=True,
        label=get_mandatory_form_label_text(text=_("Doctrine")),
    )
    aar_link = forms.CharField(required=False, label=_("After Action Report Link"))

    class Meta:  # pylint: disable=too-few-public-methods
        """
        Meta definitions
        """

        model = SrpLink
        fields = ["srp_name", "fleet_time", "fleet_type", "fleet_doctrine", "aar_link"]


class SrpLinkUpdateForm(ModelForm):
    """
    Edit SRP link update form
    """

    aar_link = forms.CharField(required=False, label=_("After Action Report Link"))

    class Meta:  # pylint: disable=too-few-public-methods
        """
        Meta definitions
        """

        model = SrpLink
        fields = ["aar_link"]


class SrpRequestForm(ModelForm):
    """
    SRP request form
    """

    killboard_link = forms.URLField(
        label=get_mandatory_form_label_text(text=_("Killboard Link")),
        max_length=254,
        required=True,
        help_text=_(
            f"Find your kill mail on {ZKILLBOARD_BASE_URL} or {EVETOOLS_KILLBOARD_BASE_URL} and paste the link here."  # pylint: disable=line-too-long
        ),
    )

    additional_info = forms.CharField(
        widget=forms.Textarea(attrs={"rows": 10, "cols": 20, "input_type": "textarea"}),
        required=True,
        label=get_mandatory_form_label_text(text=_("Additional Information")),
        help_text=_(
            "Please tell us about the circumstances of your untimely demise. "
            "Who was the FC, what doctrine was called, have changes to the fit "
            "been requested and so on. Be as detailed as you can."
        ),
    )

    class Meta:  # pylint: disable=too-few-public-methods
        """
        Meta definitions
        """

        model = SrpRequest
        fields = ["killboard_link", "additional_info"]

    def clean_killboard_link(self):
        """
        Check if it's a link from one of the accepted kill boards and clean it

        :return:
        :rtype:
        """

        killboard_link = self.cleaned_data["killboard_link"]

        # Check if it's a link from one of the accepted kill boards
        if not any(
            re.match(pattern=regex, string=killboard_link)
            for regex in [ZKILLBOARD_BASE_URL_REGEX, EVETOOLS_KILLBOARD_BASE_URL_REGEX]
        ):
            raise forms.ValidationError(
                message=_(
                    f"Invalid Link. Please use {ZKILLBOARD_BASE_URL} or {EVETOOLS_KILLBOARD_BASE_URL}"  # pylint: disable=line-too-long
                )
            )

        # Check if it's an actual killmail
        if not any(
            re.match(pattern=regex, string=killboard_link)
            for regex in [
                ZKILLBOARD_KILLMAIL_URL_REGEX,
                EVETOOLS_KILLBOARD_KILLMAIL_URL_REGEX,
            ]
        ):
            raise forms.ValidationError(
                message=_("Invalid link. Please post a link to a kill mail.")
            )

        # Check if there is already an SRP request for this kill mail
        killmail_id = SrpManager.get_kill_id(killboard_link=killboard_link)

        if SrpRequest.objects.filter(
            killboard_link__icontains="/kill/" + killmail_id
        ).exists():
            raise forms.ValidationError(
                message=_(
                    "There is already an SRP request for this killmail. "
                    "Please check if you got the right one."
                )
            )

        return killboard_link


class SrpRequestPayoutForm(forms.Form):
    """
    Change payout value
    """

    value = forms.CharField(label=_("SRP payout value"), max_length=254, required=True)


class SrpRequestRejectForm(forms.Form):
    """
    SRP request reject form
    """

    reject_info = forms.CharField(
        widget=forms.Textarea(attrs={"rows": 10, "cols": 20, "input_type": "textarea"}),
        required=True,
        label=get_mandatory_form_label_text(text=_("Reject Reason")),
        help_text=_("Please provide the reason this SRP request is rejected."),
    )


class SrpRequestAcceptForm(forms.Form):
    """
    SRP request accept form
    """

    reviser_comment = forms.CharField(
        widget=forms.Textarea(attrs={"rows": 10, "cols": 20, "input_type": "textarea"}),
        required=False,
        label=_("Comment (Optional)"),
        help_text=_("Leave a comment for the requestor"),
    )


class SrpRequestAcceptRejectedForm(forms.Form):
    """
    SRP request accept rejected form
    """

    reviser_comment = forms.CharField(
        widget=forms.Textarea(attrs={"rows": 10, "cols": 20, "input_type": "textarea"}),
        required=True,
        label=get_mandatory_form_label_text(text=_("Comment")),
        help_text=_(
            "Please provide the reason why this former rejected SRP request is now "
            "accepted."
        ),
    )


class UserSettingsForm(ModelForm):
    """
    User settings form
    """

    disable_notifications = forms.BooleanField(
        initial=False,
        required=False,
        label=_(
            "Disable notifications. "
            "(Auth and Discord, if a relevant module is installed)"
        ),
    )

    class Meta:  # pylint: disable=too-few-public-methods
        """
        Meta definitions
        """

        model = UserSetting
        fields = ["disable_notifications"]


class SettingAdminForm(forms.ModelForm):
    """
    Form definitions for the FleetType form in admin
    """

    class Meta:  # pylint: disable=too-few-public-methods
        """
        Meta
        """

        model = Setting
        fields = "__all__"
        widgets = {"default_embed_color": forms.TextInput(attrs={"type": "color"})}



"""
Versioned static URLs to break browser caches when changing the app version
"""

# Django
from django.template.defaulttags import register
from django.templatetags.static import static

# AA SRP
from aasrp import __version__


@register.simple_tag
def aasrp_static(path: str) -> str:
    """
    Versioned static URL
    Adding the app version to any static file we load through this function.
    This is to make sure to break the browser cache on app updates.

    Example: /static/myapp/css/myapp.css?ver=1.0.0

    :param path:
    :type path:
    :return:
    :rtype:
    """

    static_url = static(path)
    versioned_url = static_url + "?v=" + __version__

    return versioned_url



# Generated by Django 4.0.7 on 2022-08-04 18:06

# Django
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("aasrp", "0006_related_names"),
    ]

    operations = [
        migrations.AddField(
            model_name="aasrprequestcomment",
            name="comment_time",
            field=models.DateTimeField(null=True, blank=True),
        ),
        migrations.AlterField(
            model_name="aasrprequestcomment",
            name="comment_type",
            field=models.CharField(
                choices=[
                    ("Comment", "Comment"),
                    ("Request Added", "SRP Request Added"),
                    ("Request Information", "Additional Information"),
                    ("Reject Reason", "Reject Reason"),
                    ("Status Changed", "Status Changed"),
                    ("Reviser Comment", "Reviser Comment"),
                ],
                default="Comment",
                max_length=19,
            ),
        ),
    ]



# Generated by Django 4.0.7 on 2022-08-04 19:18

# Django
import django.db.models.deletion
import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("eveonline", "0016_character_names_are_not_unique"),
        ("aasrp", "0007_aasrprequestcomment_comment_time_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="aasrprequestcomment",
            name="new_status",
            field=models.CharField(
                blank=True,
                choices=[
                    ("Pending", "Pending"),
                    ("Approved", "Approved"),
                    ("Rejected", "Rejected"),
                ],
                default="",
                max_length=8,
            ),
        ),
        migrations.AlterField(
            model_name="aasrprequest",
            name="character",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="eveonline.evecharacter",
            ),
        ),
        migrations.AlterField(
            model_name="aasrprequestcomment",
            name="comment",
            field=models.TextField(blank=True, default=""),
        ),
        migrations.AlterField(
            model_name="aasrprequestcomment",
            name="comment_time",
            field=models.DateTimeField(
                default=django.utils.timezone.now, null=True, blank=True
            ),
        ),
    ]



"""
Testing access to AA-SRP
"""

# Third Party
from faker import Faker

# Django
from django.contrib.auth.models import Group
from django.test import TestCase
from django.urls import reverse
from django.utils import timezone
from django.utils.crypto import get_random_string

# Alliance Auth (External Libs)
from app_utils.testing import create_fake_user

# AA SRP
from aasrp.models import SrpLink

fake = Faker()


class TestAccess(TestCase):
    """
    Testing access to various views
    """

    @classmethod
    def setUpClass(cls) -> None:
        """
        Set up groups and users
        """

        super().setUpClass()
        cls.group = Group.objects.create(name="Enterprise Crew")

        # User without access
        cls.user_without_access = create_fake_user(
            character_id=1001, character_name="Wesley Crusher"
        )

        # User with basic_access
        cls.user_with_basic_access = create_fake_user(
            character_id=1002,
            character_name="Miles O'Brian",
            permissions=["aasrp.basic_access"],
        )

        # User with create_srp
        cls.user_with_create_srp = create_fake_user(
            character_id=1003,
            character_name="Worf",
            permissions=["aasrp.basic_access", "aasrp.create_srp"],
        )

        # User with manage_srp_requests
        cls.user_with_manage_srp_requests = create_fake_user(
            character_id=1004,
            character_name="James T. Kirk",
            permissions=["aasrp.basic_access", "aasrp.manage_srp_requests"],
        )

        # User with manage_srp
        cls.user_with_manage_srp = create_fake_user(
            character_id=1005,
            character_name="Jean Luc Picard",
            permissions=["aasrp.basic_access", "aasrp.manage_srp"],
        )

        cls.defaul_srp_link = SrpLink(
            srp_name="Foobar",
            fleet_time=timezone.now(),
            fleet_doctrine="Ships",
            aar_link="",
            srp_code=get_random_string(length=16),
            fleet_commander=None,
            creator=cls.user_with_create_srp,
        )
        cls.defaul_srp_link.save()

    def test_should_show_dashboard(self):
        """
        Test that a user with basic_access can see the srp module

        :return:
        :rtype:
        """

        # given
        self.client.force_login(self.user_with_basic_access)

        # when
        res = self.client.get(path=reverse("aasrp:dashboard"))

        # then
        self.assertEqual(first=res.status_code, second=200)

    def test_should_not_show_dashboard(self):
        """
        Test that a user without basic_access can't see the srp module

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_without_access)

        # when
        res = self.client.get(path=reverse("aasrp:dashboard"))

        # then
        self.assertNotEqual(first=res.status_code, second=200)
        self.assertEqual(first=res.status_code, second=302)

    def test_should_show_dashboard_with_all_srp_links(self):
        """
        Test that a user with manage_srp can see all srp links

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_with_manage_srp)

        # when
        res = self.client.get(path=reverse("aasrp:all"))

        # then
        self.assertEqual(first=res.status_code, second=200)

    def test_should_not_show_dashboard_with_all_srp_links(self):
        """
        Test that a user without manage_srp can't see all srp links

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_with_basic_access)

        # when
        res = self.client.get(path=reverse("aasrp:all"))

        # then
        self.assertNotEqual(first=res.status_code, second=200)
        self.assertEqual(first=res.status_code, second=302)

    def test_srp_link_add_with_create_srp_permission(self):
        """
        Test if a user with create_srp permission can create an SRP link

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_with_create_srp)

        # when
        res = self.client.get(path=reverse("aasrp:add_srp_link"))

        # then
        self.assertEqual(first=res.status_code, second=200)

    def test_srp_link_add_with_manage_srp_permission(self):
        """
        Test if a user with manage_srp permission can create an SRP link

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_with_manage_srp)

        # when
        res = self.client.get(path=reverse("aasrp:add_srp_link"))

        # then
        self.assertEqual(first=res.status_code, second=200)

    def test_srp_link_add_without_appropriate_permission(self):
        """
        Test that a user with basic_access permission cannot create an SRP link

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_with_basic_access)

        # when
        res = self.client.get(path=reverse("aasrp:add_srp_link"))

        # then
        self.assertNotEqual(first=res.status_code, second=200)
        self.assertEqual(first=res.status_code, second=302)

    def test_srp_link_add_without_permission(self):
        """
        Test that a user without permissions canot create an SRP link

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_without_access)

        # when
        res = self.client.get(path=reverse("aasrp:add_srp_link"))

        # then
        self.assertNotEqual(first=res.status_code, second=200)
        self.assertEqual(first=res.status_code, second=302)

    def test_srp_link_edit_with_create_srp_permission(self):
        """
        Test if a user with create_srp permission can edit an SRP link

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_with_create_srp)

        # when
        res = self.client.get(
            path=reverse(
                viewname="aasrp:edit_srp_link", args=[self.defaul_srp_link.srp_code]
            )
        )

        # then
        self.assertEqual(first=res.status_code, second=200)

    def test_srp_link_edit_with_manage_srp_permission(self):
        """
        Test if a user with manage_srp permission can edit an SRP link

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_with_manage_srp)

        # when
        res = self.client.get(
            path=reverse(
                viewname="aasrp:edit_srp_link", args=[self.defaul_srp_link.srp_code]
            )
        )

        # then
        self.assertEqual(first=res.status_code, second=200)

    def test_request_srp_with_basic_access_permission(self):
        """
        Test if a user with basic_access can open the Request SRP view

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_with_basic_access)

        # when
        res = self.client.get(
            path=reverse(
                viewname="aasrp:request_srp", args=[self.defaul_srp_link.srp_code]
            )
        )

        # then
        self.assertEqual(first=res.status_code, second=200)

    def test_request_srp_without_permission(self):
        """
        Test that a user without access cannot open the Request SRP view

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_without_access)

        # when
        res = self.client.get(
            path=reverse(
                viewname="aasrp:request_srp", args=[self.defaul_srp_link.srp_code]
            )
        )

        # then
        self.assertNotEqual(first=res.status_code, second=200)
        self.assertEqual(first=res.status_code, second=302)

    def test_srp_link_view_requests_with_manage_srp_permission(self):
        """
        Test if a user with manage_srp permission can view srp requests

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_with_manage_srp)

        # when
        res = self.client.get(
            path=reverse(
                viewname="aasrp:view_srp_requests", args=[self.defaul_srp_link.srp_code]
            )
        )

        # then
        self.assertEqual(first=res.status_code, second=200)

    def test_srp_link_view_requests_with_manage_srp_requests_permission(self):
        """
        Test if a user with manage_srp_requests permission can view srp requests

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_with_manage_srp_requests)

        # when
        res = self.client.get(
            path=reverse(
                viewname="aasrp:view_srp_requests", args=[self.defaul_srp_link.srp_code]
            )
        )

        # then
        self.assertEqual(first=res.status_code, second=200)

    def test_srp_link_view_requests_without_permission(self):
        """
        Test that a user with basic_access permission cannot view srp requests

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_with_basic_access)

        # when
        res = self.client.get(
            path=reverse(
                viewname="aasrp:view_srp_requests", args=[self.defaul_srp_link.srp_code]
            )
        )

        # then
        self.assertNotEqual(first=res.status_code, second=200)
        self.assertEqual(first=res.status_code, second=302)
